import random


class UserDomain:
    '''
    This class is used to share domain sting with other classes while performing analysis check on domain.
    '''

    def __init__(self, domain: str = None):
        '''
        initiating class with domain name if provided
        :param domain: domain name
        :type domain:str
        '''
        self.__domain = domain

    @property
    def domain(self) -> str:
        '''
        This is property used to get information about domain
        :return: domain address
        :rtype: str
        '''
        return self.__domain

    @domain.setter
    def domain(self, _domain: str):
        '''
        This is a setter function used to set domain address.
        :param _domain: domain address of webpages
        :type _domain: str
        '''
        self.__domain = _domain


class UserAgent:
    '''
    Class contains multiple user agent for making different requests on the server.
    '''

    @staticmethod
    def user_agent():
        '''
        this function reruns random user agent from list
        :return: random user agent from list
        :rtype: str
        '''
        user_agent_list = [
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:90.0) Gecko/20100101 Firefox/90.0",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/13.1.2 Safari/605.1.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/14.0.3 Safari/605.1.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/14.1.1 Safari/605.1.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/90.0.4430.212 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.101 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.106 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.164 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.77 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/92.0.4515.107 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/14.1 Safari/605.1.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/14.1.1 Safari/605.1.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/14.1.2 Safari/605.1.15",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) "
            "Version/15.0 Safari/605.1.15",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/76.0.3809.100 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/90.0.4430.212 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/90.0.4430.93 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.101 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.106 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36 Edg/91.0.864.59",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36 OPR/77.0.4054.172",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36 Edg/91.0.864.64",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36 Edg/91.0.864.67",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36 Edg/91.0.864.70",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36 OPR/77.0.4054.203",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.164 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.164 Safari/537.36 Edg/91.0.864.71",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.77 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/92.0.4515.107 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/92.0.4515.107 Safari/537.36 Edg/92.0.902.55",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:88.0) Gecko/20100101 Firefox/88.0",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0",
            "Mozilla/5.0 (Windows NT 10.0; rv:78.0) Gecko/20100101 Firefox/78.0",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.93 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.101 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.114 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.164 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.77 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64; rv:78.0) Gecko/20100101 Firefox/78.0",
            "Mozilla/5.0 (X11; Linux x86_64; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (X11; Linux x86_64; rv:90.0) Gecko/20100101 Firefox/90.0",
            "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:89.0) Gecko/20100101 Firefox/89.0",
            "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:90.0) Gecko/20100101 Firefox/90.0"
        ]
        _user_agent = random.choice(user_agent_list)
        return str(_user_agent)



from domaintrails.core.resources import UserDomain

from domaintrails.core.recon.domain_available import DomainAvailable as DomainAvailability
from domaintrails.core.recon.dns_history import DnsHistory as DomainHostHistory
from domaintrails.core.parser import ColorPrint
from domaintrails.core.recon.dnslg import DnsLG as DomainDnsInformation
from domaintrails.core.resources import UserAgent


def index_in_list(a_list: list, index: int):
    '''
    This function is to check if some index exists inside a list.
    :param a_list: List of items
    :type a_list: list
    :param index: index pointer which needs be checked on list
    :type index: int
    :return: True / False
    :rtype: bool
    '''
    return index < len(a_list)


def print_table(table_data: list):
    '''
    We use this function to create table for printing information on console / terminal. We will be calculating
    column's information, separator and designing layout. After calculation this function will print information
    on terminal.
    :param table_data: table data in shape of list
    :type table_data: list
    '''

    sep_index = []
    for row in table_data:
        l_row = len(row)
        ptr_spe_index = 0
        while l_row:
            l_row = l_row - 1
            col_width = len(str(row[l_row])) + 2
            if index_in_list(sep_index, ptr_spe_index):
                stored_value = sep_index[ptr_spe_index]
                current_value = col_width
                if current_value > stored_value:
                    stored_value = current_value
                sep_index[ptr_spe_index] = stored_value
            else:
                sep_index.insert(
                    ptr_spe_index, col_width)

            ptr_spe_index = ptr_spe_index + 1
    separator_info = list(reversed(sep_index))
    separator_list = []
    for ptr_spe in separator_info:
        separator_list.append(
            "-" * (ptr_spe - 2)
        )

    table_data.insert(
        1, separator_list
    )
    for row in table_data:

        ptr_spe = 0
        output_line = ""
        for word in row:
            output_line = output_line + str(word).ljust(separator_info[ptr_spe])
            ptr_spe = ptr_spe + 1

        print(output_line)


def dict_to_table(dict_data: dict):
    '''
    This function converts dictionary data into list. This function is normally used to prepare data before printing
    on console/ terminal.
    :param dict_data: dictionary based data from different sources
    :type dict_data: dict
    :return: list based data is returned to caller
    :rtype: list
    '''
    table_data = []
    t_keys = []
    for row in dict_data:
        t_keys = list(row.keys())
        table_data.append(list(row.values()))
    update_key = []
    for key in t_keys:
        update_key.append(str(key).title().replace("_", " "))
    table_data.insert(0, t_keys)
    return table_data


def process_request(domain: str = None):
    '''
    This function is responsible for calling all the configured module and services in project. Following services are
    called by the function,
    1. DomainAvailability
    2. DomainHostHistory
    3. DomainDnsInformation

    Function later calls printing function to display all content

    '''
    if domain:
        UserDomain.domain = domain
    domain = UserDomain.domain

    ColorPrint.print_bold("Printing Results for domain %s" % domain)

    is_domain_available = DomainAvailability(str(domain))
    response = is_domain_available.domain_available
    if isinstance(response, bool):
        if response:
            _info = "Domain Available"
        else:
            _info = "Domain is Registered"
    else:
        _info = "unable to find result."

    ColorPrint.print_info("Domain Availability Result for %s : %s" % (domain, _info))

    dns_history = DomainHostHistory(str(domain))
    __domain_history = dns_history.domain_history()

    if __domain_history:
        ColorPrint.print_info("DNS History Records")
        table_data = [
            [
                "Old Web Host", "New Web Host", "Month / Year", "Zone Date", "Transaction"
            ]
        ]
        for row in __domain_history:
            table_data.append(
                [
                    row["old_web_host"],
                    row["new_web_host"],
                    row["month_year"],
                    row["zone_date"],
                    row["transaction"],
                ]
            )
        print_table(table_data)

    dns_information = DomainDnsInformation(str(domain))
    dns_records = dns_information.download_report()

    for dns_record in dns_records:
        try:
            record = dns_records[dns_record]
            ColorPrint.print_info(record["title"])

            records = record["records"]
            print_table(dict_to_table(records))
        except TypeError:
            # @TODO: Check which kind of records are failing while converting into table and printing process
            pass


__all__ = ["UserDomain", "UserAgent", "process_request", "DomainHostHistory"]



import sys


class UrlParser:
    """
    Class addresses cosmetic settings for an url. Different online services requires different shape of url so
    this class generally contains different function which beautify url/domain according to its requirement.
    """

    def __init__(self):
        """
        Setting url value to None
        """
        self.__url = None

    @property
    def url(self) -> str:
        """
        it returns url which was set before
        :return: domain / url
        :rtype: str
        """

        return self.__url

    @url.setter
    def url(self, u_url: str) -> None:
        """
        setter for url
        :param u_url: url as input of function
        :type u_url: str
        """
        self.__url = u_url

    @property
    def root_url(self) -> str:
        """
        This property function extracts root domain from given url

        :return: domain e.g. www.google.com
        :rtype: str
        """
        url = self.__url

        if str(url).startswith("https://"):
            url = str(url).replace("https://", "")
        if str(url).startswith("http://"):
            url = str(url).replace("http://", "")
        if ":" in url:
            content = str(url).split(":")
            if len(content) > 1:
                url = content[0]
            else:
                url = content
        if "/" not in url:
            return url
        url = url.split("/")
        if len(url) > 1:
            return url[0]
        return url[0]


url_parser = UrlParser()


def domain_for_md(domain: str) -> str:
    """
    This function prepares domain for domain availability checking service.
    :param domain: takes input in any shape of urls e.g. http://www.example.com, https://www.example.com:8080
    :type domain: str
    :return: root of domain e.g. www.example.com
    :rtype: str
    """
    url_parser.url = domain
    domain = url_parser.root_url
    return domain


def domain_for_history(domain: str) -> str:
    """
    This function prepares domain for domain history checking service.
    :param domain: takes input in any shape of urls e.g. http://www.example.com, https://www.example.com:8080
    :type domain: str
    :return: simplified  domain e.g. example.com
    :rtype: str
    """
    url_parser.url = domain
    domain = url_parser.root_url
    if domain.startswith("www."):
        domain = domain.replace("www.", "")
    return domain


class ColorPrint:
    """
    This class provides support for color printing text in console.
    """

    @staticmethod
    def print_fail(message: str, end: str = '\n') -> None:
        """
        Print text in red color in console
        :param message: message which needs to be printed on console.
        :type message: str
        :param end: new line
        :type end: str
        :return: None
        :rtype: None
        """
        sys.stderr.write('\x1b[1;31m' + message.strip() + '\x1b[0m' + end)

    @staticmethod
    def print_pass(message: str, end: str = '\n') -> None:
        """
        Print text in green color in console
        :param message: message which needs to be printed on console.
        :type message: str
        :param end: new line
        :type end: str
        :return: None
        :rtype: None
        """

        sys.stdout.write('\x1b[1;32m' + message.strip() + '\x1b[0m' + end)

    @staticmethod
    def print_warn(message: str, end: str = '\n') -> None:
        """
        Print text in warning color in console
        :param message: message which needs to be printed on console.
        :type message: str
        :param end: new line
        :type end: str
        :return: None
        :rtype: None
        """
        sys.stderr.write('\x1b[1;33m' + message.strip() + '\x1b[0m' + end)

    @staticmethod
    def print_info(message: str, end: str = '\n') -> None:
        """
        Print text in green color in console
        :param message: message which needs to be printed on console.
        :type message: str
        :param end: new line
        :type end: str
        :return: None
        :rtype: None
        """
        sys.stdout.write('\x1b[1;34m' + "\n" + message.strip() + "\n" + '\x1b[0m' + end)

    @staticmethod
    def print_bold(message: str, end: str = '\n') -> None:
        """
        Print text in bold format in console
        :param message: message which needs to be printed on console.
        :type message: str
        :param end: new line
        :type end: str
        :return: None
        :rtype: None
        """
        sys.stdout.write('\x1b[1;37m' + message.strip() + '\x1b[0m' + end)


__all__ = [ "domain_for_history", "ColorPrint", "domain_for_md"]



import os
import argparse
import time

from domaintrails.core.resources import UserDomain
from domaintrails.core.parser import ColorPrint
from domaintrails.core import process_request as ProcessRequest


def banner():
    """
    banner function is showing ascii art for project name. It contains Domain Trails and tag line for project
    :return: strings for project banner.
    :rtype: basestring
    """

    banner_text = """
  -------------------------------------------------------------------------------
      ____                            _           ______              _  __     
     / __ \ ____   ____ ___   ____ _ (_)____     /_  __/_____ ____ _ (_)/ /_____
    / / / // __ \ / __ `__ \ / __ `// // __ \     / /  / ___// __ `// // // ___/
   / /_/ // /_/ // / / / / // /_/ // // / / /    / /  / /   / /_/ // // /(__  ) 
  /_____/ \____//_/ /_/ /_/ \__,_//_//_/ /_/    /_/  /_/    \__,_//_//_//____/  
  -------------------------------------------------------------------------------
    Domain Trails - domains footprints, reconnaissance & information gathering 
  -------------------------------------------------------------------------------                                                                              
    """
    return banner_text


def main(domain: str = None) -> None:
    if not domain:
        os.system("")
        ColorPrint.print_warn(banner())
        # Just giving small amount of sleep before printing content on screen
        time.sleep(.5)
        parser = argparse.ArgumentParser(
            description="Domain Trails, a project for domain information gathering from online"
                        "sources.")
        parser.add_argument("-d", dest="domain", required=True, type=str, help="Domain address for recon operation.")
        # parser.add_argument("-p", dest="enable_proxy", type=bool, default=False,
        #                     help="Enable free ssl proxies while performing analysis.")
        args = parser.parse_args()

        if args.domain:
            UserDomain.domain = args.domain
    else:
        UserDomain.domain = domain
    ProcessRequest()


if __name__ == '__main__':
    main()



import json
from urllib.error import HTTPError

from domaintrails.core.resources import UserDomain, UserAgent
from domaintrails.core.parser import domain_for_md as UrlParser
from urllib.request import urlopen, Request
from urllib.parse import urlencode


class DomainAvailable:
    '''
    This class check if a domain is available or not. We are using mdchecker.com for collecting information
    from server.
    '''
    __ENDPOINT__ = "https://madchecker.com/api/domain/get-information"

    def __init__(self, domain: str = None):
        '''
        Init function for data initialization
        :param domain: domain name required for analysis
        :type domain: str
        '''
        if domain:
            self.__domain = domain
        else:
            self.__domain = UserDomain.domain

        self.__is_available = False

    @property
    def domain_available(self) -> bool:
        '''
        This function check domain availability from server, and return result.
        :return: True/ False / raise exception in case of failure
        :rtype: bool
        '''
        domain = UrlParser(self.__domain)
        params = {
            "domain": domain
        }
        query_string = urlencode(params)
        endpoint_url = self.__ENDPOINT__ + "?" + query_string
        try:
            user_agent = UserAgent()
            endpoint_request = Request(
                endpoint_url,
                None,
                headers={
                    'User-Agent': str(user_agent.user_agent()),
                }
            )
            with urlopen(endpoint_request) as response:
                response_text = response.read()
                content = json.loads(response_text)
                if "domain" in content:
                    domain_details = content["domain"]
                    available = domain_details["available"]
                    if available:
                        return True
                    else:
                        return False
                # @TODO: Resolve this exception with tuple and let the main function decide
                raise KeyError("We are failed to find domain status from server. Please try again in few minutes.")
        except HTTPError as e:
            raise e



import urllib
from urllib.request import Request, urlopen
import urllib.parse

from domaintrails.core.resources import UserAgent, UserDomain

from domaintrails.core.parser.html_parser import DnsHistoryParser as HistoryWebParser
from domaintrails.core.parser import domain_for_history as UrlParser


class DnsHistory:
    '''
    This class is used to collect information from server related to dns history, we are using
    http://www.hosterstats.com service for grabbing records.
    '''
    __END_POINT__ = 'http://www.hosterstats.com/historicaldns.php'

    def __init__(self, domain: str = None):
        '''
        init function
        :param domain: domain name as input
        :type domain: str
        '''
        if domain:
            self.__domain = domain
        else:
            self.__domain = UserDomain.domain

    def domain_history(self) -> list:
        '''
        This function download history from server for given domain name
        :return: function returns list of records as part of its output
        :rtype: list
        '''
        domain = UrlParser(self.__domain)
        params = {
            "domain": domain
        }
        query_string = urllib.parse.urlencode(params)
        endpoint_url = self.__END_POINT__ + "?" + query_string
        request = Request(endpoint_url)
        user_agent = UserAgent()
        request.add_header('Connection', 'keep-alive')
        request.add_header('Cache-Control', 'max-age=0')
        request.add_header('DNT', '1')
        request.add_header('Upgrade-Insecure-Requests', '1')
        request.add_header('Accept-Language', 'en-US,en;q=0.9')
        request.add_header('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,'
                                     'image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9')
        request.add_header('User-Agent', str(user_agent.user_agent()))

        try:
            content = urlopen(request).read()
            history = HistoryWebParser(content)
            report = history.parse()

        except Exception:
            return []
        return report



import re

from typing import Tuple


class DnsHistoryParser:
    """
    Custom parser written for extracting data from html.
    """

    def __init__(self, html_content: str):
        """
        initiating function with some html content data
        :param html_content: html data
        :type html_content: str
        """
        self.__html = html_content

    @staticmethod
    def __ext_table_data__(web_content: str) -> Tuple[list, int]:
        """
        This function extracts tables from html content, it creates a list and append all extracted tables in it.
        :param web_content: string based html content
        :type web_content:  str
        :return html_tables, max_table_length: list of extracted tables, number of tables in table list
        :rtype html_tables, max_table_length: list, int
        """
        web_content = str(web_content)
        html_tables = []
        max_table_length = 0
        html_table_regex = r'<table.*?/table>'
        table_row_regex = r'<tr.*?/tr>'
        row_data_regex = r'<(td|th).*?/(td|th)>'
        found_tables = re.search(html_table_regex, web_content, re.DOTALL)
        while found_tables:
            table_record = found_tables.group()
            found_table_rows = re.search(table_row_regex, table_record, re.DOTALL)
            __table_record = []
            while found_table_rows:
                record_row = found_table_rows.group()  # the row
                record_row_data = re.search(row_data_regex, record_row, re.DOTALL)
                __row_record = []
                while record_row_data:
                    record_row_cell_data = record_row_data.group()
                    __row_record.append(record_row_cell_data.strip())
                    record_row = re.sub(row_data_regex, '', record_row, 1, re.DOTALL)
                    record_row_data = re.search(row_data_regex, record_row, re.DOTALL)
                __table_record.append(__row_record)
                if max_table_length < len(__row_record):
                    max_table_length = len(__row_record)
                table_record = re.sub(table_row_regex, '', table_record, 1, re.DOTALL)
                found_table_rows = re.search(table_row_regex, table_record, re.DOTALL)

            web_content = re.sub(html_table_regex, '', web_content, 1, re.DOTALL)
            html_tables.append(__table_record)
            found_tables = re.search(html_table_regex, web_content, re.DOTALL)
        return html_tables, max_table_length

    def parse(self) -> list:
        """
        This functions invokes parsing of html content
        :return: historical records against website
        :rtype: object
        """
        extracted_html_tables, max_len = self.__ext_table_data__(self.__html)
        history_data = []
        if len(extracted_html_tables) > 0:
            timeline_table = extracted_html_tables[0]
            if isinstance(timeline_table, list):
                if "<th scope=\"col\">Old Hoster</th>".lower() in str(timeline_table).lower():
                    for history_record in timeline_table[1:]:
                        extract_row_record = []
                        for record_column in history_record:
                            transaction_value = re.findall(r'>(.+?)<', record_column)
                            if len(transaction_value) == 1:
                                transaction_value = str(transaction_value[0]).replace("<center>", "")
                                transaction_value = str(transaction_value).replace("&nbsp;", " ")
                                if 'N/A' in transaction_value:
                                    transaction_value = "N/A"
                                elif transaction_value.lower() == "transfer":
                                    transaction_value = "Transfer"
                                elif transaction_value.lower() == "added":
                                    transaction_value = "Added"
                                elif transaction_value.lower() == "new":
                                    transaction_value = "New"
                                elif transaction_value.lower() == "removed":
                                    transaction_value = "Removed"
                                elif transaction_value.lower() == "deleted":
                                    transaction_value = "Deleted"
                                elif transaction_value.lower() == "epoch":
                                    transaction_value = "Epoch"
                                extract_row_record.append(
                                    transaction_value
                                )
                            else:
                                extract_row_record.append(
                                    ""
                                )
                        if extract_row_record:
                            host_record = {
                                "old_web_host": str(extract_row_record[0]).lower().rstrip().strip(),
                                "new_web_host": str(extract_row_record[1]).lower().rstrip().strip(),
                                "month_year": str(extract_row_record[2]).lower().rstrip().strip(),
                                "zone_date": str(extract_row_record[3]).lower().rstrip().strip(),
                                "transaction": str(extract_row_record[4]).lower().rstrip().strip(),
                            }
                            history_data.append(host_record)

        return history_data



import json
from urllib.error import HTTPError
from urllib.request import Request, urlopen
from domaintrails.core.resources import UserDomain, UserAgent

from domaintrails.core.parser import domain_for_history as UrlParser
import concurrent.futures


class DnsLG:
    '''
    Information:
    This class collect information about all the dns services currently available for domain. Following is the list of
    dns classes which are covered as part of this dns,
    Classes
    a       -> Host Address (A records)
    cert    -> Certificate (CERT records)
    dhcid     -> DHCP Identifier (DHCID records)
    cname     -> Canonical Name (CNAME records)
    aaaa    -> Pv6 Host Address (AAAA records)
    dlv     -> DNSSEC Lookaside Validation record (DLV records)
    dname     -> Delegation name (DNAME records)
    dnskey    -> DNS Key record (DNSKEY records)
    ds      -> Delegation Signer (DS records)
    hinfo     -> Host Information (HINFO records)
    hip     -> Host Identity Protocol (HIP records)
    kx      -> Key eXchanger record (KX records)
    loc     -> Location record (LOC records)
    mx      -> Mail Exchange record (MX records)
    naptr     -> Name Authority Pointer (NAPTR records)
    ns      -> Name Servers (NS records)
    nsec     -> Next-Secure record (NSEC records)
    nsec3     -> NSEC record version 3 (NSEC3 records)
    nsec3param-> NSEC3 parameters (NSEC3PARAM records)
    opt     -> Option record (OPT records)
    talink    -> Trust Anchor LINK (TALINK records)
    tlsa    -> TLSA records
    txt     -> Text record (TXT records)
    ta      -> DNSSEC Trust Authorities (TA records)
    rrsig     -> Resource Records Signature (RRSIG records)
    soa     -> Start of Authority (SOA record)
    spf     -> Sender Policy Framework (SPF records)
    srv     -> Service Locator (SRV records)
    sshfp     -> SSH Public Key Fingerprint (SSHFP records)
    above classes are part of this service.
    '''

    __get_auth = False
    __accept_value = 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avi' \
               'f,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9'

    def __init__(self, domain: str = None, get_authority: bool = False):
      '''
      Initiating and assigning dns object with domain information.
      :param domain: domain which will be analyzed for records collection.
      :type domain: str
      :param get_authority: want to store authority information, default disabled
      :type get_authority: bool
      '''
      self.__get_auth = get_authority
      if domain:
        self.__domain = UrlParser(domain)
      else:
        self.__domain = UrlParser(str(UserDomain.domain))

    def __add_auth__(self, content: dict, result: list) -> list:
      '''
      This function will extract authority information from json response. It will store only when
      function is set self.__get_auth=True else it will be ignored
      :param content: dict of records downloaded from server
      :type content:  dict
      :param result: list of records shared with function, data is stored in this list
      :type result: list
      :return: list of records
      :rtype: list
      '''
      if self.__get_auth:
        if "authority" in content:
            authorities = content["authority"]
            for authority in authorities:
              record = {
                "name": authority["name"],
                "type": authority["type"],
                "class": authority["class"],
                "ttl": authority["ttl"],
                "endpoint": self.__extract_rdata__(authority["rdata"]),
              }
              if record not in result:
                result.append(record)
      return result

    def __add_answer__(self, content: dict) -> list:
      '''
      This function will parse the response from server. If we are answers against the query we will pass
      information for processing.
      :param content: dict content downloaded from server against domain
      :type content: dict
      :return: list of records
      :rtype: list
      '''
      result = []
      if "answer" in content:
        answers = content["answer"]
        if answers:
            for answer in answers:
              r_data = self.__extract_rdata__(answer["rdata"])
              if len(r_data) == 1:
                r_data = r_data[0]
              else:
                r_data = r_data
              record = {
                "name": answer["name"],
                "type": answer["type"],
                "class": answer["class"],
                "ttl": answer["ttl"],
                "endpoint": r_data,
              }
              if record not in result:
                result.append(record)
      return result

    @staticmethod
    def __extract_rdata__(rdata: str) -> list:
      '''
      This function extract data from strings and make it readable in lists for different classes.
      :param rdata: class endpoint data
      :type rdata: str
      :return: list of records found in endpoint
      :rtype: list
      '''
      contents = str(rdata).split(" ")
      if len(contents) > 1:
        if "v=spf" in contents[0].lower() or len(contents) == 2:
            return [rdata]
        elif len(contents) == 7:
            return [{
              "primary_nameserver": contents[0],
              "host_master_email": contents[1],
              "serial_number": contents[2],
              "refresh": contents[3],
              "retry": contents[4],
              "expire": contents[5],
              "minimum_ttl": contents[6],
            }]
        else:
            # @TODO: Need to parse this type of classes for table
            pass

        return contents
      else:
        contents = [contents[0].replace('"', "")]
        return contents

    def __download_record__(self, endpoint, timeout=10):
      '''

      :param endpoint:
      :type endpoint:
      :param timeout:
      :type timeout:
      :return:
      :rtype:
      '''
      request = Request(endpoint)
      user_agent = UserAgent()
      request.add_header('Connection', 'keep-alive')
      request.add_header('Cache-Control', 'max-age=0')
      request.add_header('DNT', '1')
      request.add_header('Upgrade-Insecure-Requests', '1')
      request.add_header('Accept-Language', 'en-US,en;q=0.9')
      request.add_header('Accept', self.__accept_value)
      request.add_header('User-Agent', str(user_agent.user_agent()))
      try:
        content = urlopen(request, timeout=timeout).read()
        content = json.loads(content)
        result = self.__add_answer__(content)
        result = self.__add_auth__(content, result)
      except HTTPError:
        return {}
      return result

    def download_report(self) -> dict:
      '''
      This function create different threads by using multiprocessing, and it performs multiple api request from
      server, response is collected and parsed according usage. This function also slices data as per requirements
      we are ignoring different classes from api response.
      :return: a result dict is returned which contains records
      :rtype: dict
      '''
      dns_info = {
      }
      endpoints = [
        "http://www.dns-lg.com/us01/" + self.__domain + "/a ",
        "http://www.dns-lg.com/us01/" + self.__domain + "/cert",
        "http://www.dns-lg.com/us01/" + self.__domain + "/dhc" + "id",
        "http://www.dns-lg.com/us01/" + self.__domain + "/cname",
        "http://www.dns-lg.com/us01/" + self.__domain + "/aa" + "aa",
        "http://www.dns-lg.com/us01/" + self.__domain + "/dlv",
        "http://www.dns-lg.com/us01/" + self.__domain + "/d" + "name",
        "http://www.dns-lg.com/us01/" + self.__domain + "/dns" + "key",
        "http://www.dns-lg.com/us01/" + self.__domain + "/ds",
        "http://www.dns-lg.com/us01/" + self.__domain + "/h" + "info",
        "http://www.dns-lg.com/us01/" + self.__domain + "/hip",
        "http://www.dns-lg.com/us01/" + self.__domain + "/kx",
        "http://www.dns-lg.com/us01/" + self.__domain + "/loc",
        "http://www.dns-lg.com/us01/" + self.__domain + "/mx",
        "http://www.dns-lg.com/us01/" + self.__domain + "/na" + "ptr",
        "http://www.dns-lg.com/us01/" + self.__domain + "/ns",
        "http://www.dns-lg.com/us01/" + self.__domain + "/opt",
        "http://www.dns-lg.com/us01/" + self.__domain + "/n" + "sec3",
        "http://www.dns-lg.com/us01/" + self.__domain + "/n" + "sec3param",
        "http://www.dns-lg.com/us01/" + self.__domain + "/ta" + "link",
        "http://www.dns-lg.com/us01/" + self.__domain + "/t" + "lsa",
        "http://www.dns-lg.com/us01/" + self.__domain + "/txt",
        "http://www.dns-lg.com/us01/" + self.__domain + "/ta",
        "http://www.dns-lg.com/us01/" + self.__domain + "/rr" + "sig",
        "http://www.dns-lg.com/us01/" + self.__domain + "/soa",
        "http://www.dns-lg.com/us01/" + self.__domain + "/spf",
        "http://www.dns-lg.com/us01/" + self.__domain + "/srv",
        "http://www.dns-lg.com/us01/" + self.__domain + "/ssh" + "fp",
        "http://www.dns-lg.com/us01/" + self.__domain + "/n" + "sec"
      ]
      data = []
      with concurrent.futures.ThreadPoolExecutor(10) as executor:
        future_to_url = {executor.submit(self.__download_record__, url): url for url in endpoints}
        for future in concurrent.futures.as_completed(future_to_url):
            url = future_to_url[future]
            try:
              result = future.result()
              if result:
                data.append(result)
            except Exception as exc:
              print('%r generated an exception: %s' % (url, exc))
      for _record in data:
        for record in _record:
            if record["type"].lower() == "a":
              if "a" not in dns_info:
                dns_info["a"] = {
                    "title": "Parent Name Server records",
                    "records": [],
                }
              dns_info["a"]["records"].append(record)
            elif record["type"].lower() == "aaaa":
              if "aaaa" not in dns_info:
                dns_info["aaaa"] = {
                    "title": "IP v6 Records",
                    "records": [],
                }
              dns_info["aaaa"]["records"].append(record)

            elif record["type"].lower() == "ns":
              if "ns" not in dns_info:
                dns_info["ns"] = {
                    "title": "Local Name Server Records",
                    "records": [],
                }
              dns_info["ns"]["records"].append(record)
            elif record["type"].lower() == "mx":
              if "mx" not in dns_info:
                dns_info["mx"] = {
                    "title": "Mail eXchanger (MX) Records",
                    "records": [],
                }
              dns_info["mx"]["records"].append(record)
            elif record["type"].lower() == "soa":
              if "soa" not in dns_info:
                dns_info["soa"] = {
                    "title": "Start of Authority (SOA)",
                    "records": [],
                }
              dns_info["soa"]["records"].append(record["endpoint"])
            elif record["type"].lower() == "txt":
              if "txt" not in dns_info:
                dns_info["txt"] = {
                    "title": "Text Records",
                    "records": [],
                }
              dns_info["txt"]["records"].append(record)
            else:
              # @TODO: Handle other famous classes for table output
              pass
      return dns_info



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
version = "1.0.0"
packages = [
    "domaintrails",
    "domaintrails.core",
    "domaintrails.core.parser",
    "domaintrails.core.recon",

]
package_data = {
    'domaintrails': [
        'README.html',
        'LICENSE.txt',
    ]
}

entry_points = {
    'console_scripts': [
        'domaintrails=domaintrails.portal:main',
    ],
}

setuptools.setup(
    name="domain-trails",
    version=version,
    author="Muhammad Abdullah",
    author_email="iamabdullahmughal@gmail.com",
    description="Domain trails is open source intelligence gathering tool. This tool collect information "
                "related to domain. ",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/iAbdullahMughal/domain-trails",
    project_urls={
        "Bug Tracker": "https://github.com/iAbdullahMughal/domain-trails/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=packages,
    package_data=package_data,
    entry_points=entry_points,
    python_requires=">=3.6",
)
